<?php
    /*
        JSON Viewer 1.2
        Copyright (c) 2008-2009, Hay Kranen <http://www.haykranen.nl/projects/jsonviewer/>
        
        == Release notes ==
        
        === 1.2 ===
        * First node of the tree is now always shown
        * Clicking on the url only deletes the complete field if it is the default url
        
        === 1.1 ===
        * Fixed parsing bug that makes the viewer more flexible with feeds that
          are not valid
        
        === 1.0 ===
        * First release
        
        Released under the MIT license, reproduced here below
        
        Permission is hereby granted, free of charge, to any person
        obtaining a copy of this software and associated documentation
        files (the "Software"), to deal in the Software without
        restriction, including without limitation the rights to use,
        copy, modify, merge, publish, distribute, sublicense, and/or sell
        copies of the Software, and to permit persons to whom the
        Software is furnished to do so, subject to the following
        conditions:
        
        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.
        
        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
        OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
        NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
        HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
        WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
        FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
        OTHER DEALINGS IN THE SOFTWARE.
    */    
    
    function make_tree($var) {
        global $tree;
        foreach ($var as $key => $value) {
            if (is_array($value)) {
                $tree .= '<li><img src="arrow.png" class="arrow" alt="+" />'.$key."\t<ul>";
                make_tree($value);
            } else {
                $tree .= '<li><img src="mark.png" alt="-" />'.$key."<br />$value</li>\n";                
            }
        }
        $tree .= "</ul></li>";
        return $tree;
    }
        
    function json_viewer($url) {
        $json = @file_get_contents($url);
        if (!$json) return "Could not read data from '$url'";
        
        // Some API's deliver invalid JSON, such as Flickr and Google Suggest, 
        // usually because they deliver the JSON as a JavaScript variable, e.g.
        // 'JSONResult = ({"foo": "bar"});
        // We can 'fix' this by stripping that out before and after 
        // the { and [ characters. Unfortunately that takes a little bit of 
        // extra code, and we are essentialy fixing things that Google or Yahoo
        // should fix, but because these feeds are so popular we do it anyway.
        
        // get first occurence of curly brace and square brace
        $curly  = strpos($json, "{");
        $square = strpos($json, "[");
        
        // No curly or square bracket means this is not JSON data
        if ( ($curly === false) && ($square === false) ) {
            return "Invalid JSON data (no '{' or '[' found)";
        } else {
            // There is a case when you have a feed with [{
            // so get the first one
            if (($curly !== false) && ($square !== false)) {
                if ($curly < $square) {
                    $square = false;
                } else {
                    $curly  = false;
                }
            }
            
            // get the last curly or square brace
            if($curly !== false) {
                $firstchar = $curly;
                $lastchar  = strrpos($json, "}");
            } else if ($square !== false) {
                $firstchar = $square;
                $lastchar  = strrpos($json, "]");
            }
            if ($lastchar === false) return "Invalid JSON data (no closing '}' or ']' found)";
            
            // Give warning if $firstchar is not the first character
            if ($firstchar > 0) {
                $warning  = "---WARNING---\n";
                $warning .= "Invalid JSON data that does not begin with '{' or '[' might give unexpected results\n";
            }
        }
        // get the JSON data between the first and last curly or square brace
        $json = substr($json, $firstchar, ($lastchar - $firstchar) + 1);
        
        // decode json data
        $data = json_decode($json, true);        
        if (!$data) return "Invalid JSON data (could not decode JSON data)";
        
        if (isset($warning)) echo $warning;
    
        // we need to make the first 'root' tree element
        $out  = '<ul id="root"><li><img src="arrow.png" class="arrow" alt="+" />ROOT<ul id="first">';
        $out .= make_tree($data);
        $out .= "</ul></li></ul>";
        
        $tree = '';
        return $out;
    }
    
    // call function
    if (isset($_POST['url'])) {
        $tree = '';        
        $url = $_POST['url'];
        if ($url) echo json_viewer($url);
    }
?>